=== modified file 'NEWS'
--- NEWS	2006-09-17 21:03:04 +0000
+++ NEWS	2006-09-20 20:59:27 +0000
@@ -35,6 +35,10 @@
       which processes these requests. (Andrew Bennetts, Robert Collins, Martin
       Pool)
 
+    * Added help topics to the bzr help system
+
+    * On line documentation of the revision spec (see bzr help revisionspec)
+
   BUG FIXES:
 
     * Don't access e.code for generic URLErrors, only HTTPErrors have .code.

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2006-09-18 09:58:40 +0000
+++ bzrlib/builtins.py	2006-09-20 20:59:27 +0000
@@ -2324,6 +2324,7 @@
         else:
             return 0
 
+
 class cmd_revert(Command):
     """Reverse all changes since the last commit.
 
@@ -2370,23 +2371,25 @@
 class cmd_help(Command):
     """Show help on a command or other topic.
 
-    For a list of all available commands, say 'bzr help commands'."""
+    For a list of all available commands, say 'bzr help commands'.
+    """
     takes_options = [Option('long', 'show help on all commands')]
     takes_args = ['topic?']
     aliases = ['?', '--help', '-?', '-h']
     
     @display_command
     def run(self, topic=None, long=False):
-        import help
+        import bzrlib.help
         if topic is None and long:
             topic = "commands"
-        help.help(topic)
+        bzrlib.help.help(topic)
 
 
 class cmd_shell_complete(Command):
     """Show appropriate completions for context.
 
-    For a list of all available commands, say 'bzr shell-complete'."""
+    For a list of all available commands, say 'bzr shell-complete'.
+    """
     takes_args = ['context?']
     aliases = ['s-c']
     hidden = True
@@ -2400,7 +2403,8 @@
 class cmd_fetch(Command):
     """Copy in history from another branch but don't merge it.
 
-    This is an internal method used for pull and merge."""
+    This is an internal method used for pull and merge.
+    """
     hidden = True
     takes_args = ['from_branch', 'to_branch']
     def run(self, from_branch, to_branch):
@@ -2413,7 +2417,8 @@
 class cmd_missing(Command):
     """Show unmerged/unpulled revisions between two branches.
 
-    OTHER_BRANCH may be local or remote."""
+    OTHER_BRANCH may be local or remote.
+    """
     takes_args = ['other_branch?']
     takes_options = [Option('reverse', 'Reverse the order of revisions'),
                      Option('mine-only', 

=== modified file 'bzrlib/help.py'
--- bzrlib/help.py	2006-09-08 18:46:29 +0000
+++ bzrlib/help.py	2006-09-20 20:59:27 +0000
@@ -20,47 +20,28 @@
 # executable files with reasonable names.
 
 # TODO: `help commands --all` should show hidden commands
-import textwrap
-
-global_help = \
-"""Bazaar -- a free distributed version-control tool
-http://bazaar-vcs.org/
-
-Basic commands:
-
-  bzr init           makes this directory a versioned branch
-  bzr branch         make a copy of another branch
-
-  bzr add            make files or directories versioned
-  bzr ignore         ignore a file or pattern
-  bzr mv             move or rename a versioned file
-
-  bzr status         summarize changes in working copy
-  bzr diff           show detailed diffs
-
-  bzr merge          pull in changes from another branch
-  bzr commit         save some or all changes
-
-  bzr log            show history of changes
-  bzr check          validate storage
-
-  bzr help init      more help on e.g. init command
-  bzr help commands  list all commands
-"""
-
 
 import sys
 
-
-def help(topic=None, outfile = None):
+from bzrlib import help_topics
+
+
+help_topics.add_topic("commands",
+                      (lambda name, outfile: help_commands(outfile)),
+                      "Basic help for all commands")
+
+
+def help(topic=None, outfile=None):
+    """Print out the help for a specific topic or command."""
+
     if outfile is None:
         outfile = sys.stdout
     if topic is None:
-        outfile.write(global_help)
-    elif topic == 'commands':
-        help_commands(outfile = outfile)
+        help_topics.write_topic("basic", outfile)
+    elif help_topics.is_topic(topic):
+        help_topics.write_topic(topic, outfile)
     else:
-        help_on_command(topic, outfile = outfile)
+        help_on_command(topic, outfile=outfile)
 
 
 def command_usage(cmd_object):
@@ -164,4 +145,4 @@
         if cmd_help:
             firstline = cmd_help.split('\n', 1)[0]
             print >>outfile, '        ' + firstline
-        
+

=== modified file 'bzrlib/revisionspec.py'
--- bzrlib/revisionspec.py	2006-09-03 15:58:15 +0000
+++ bzrlib/revisionspec.py	2006-09-20 20:59:27 +0000
@@ -235,6 +235,23 @@
 # private API
 
 class RevisionSpec_revno(RevisionSpec):
+    """Selects a revision using a number.
+
+    Use an integer to specify a revision in the history of the branch.
+    Optionally a branch can be specified. The 'revno:' prefix is optional.
+    A negative number will count from the end of the branch (-1 is the
+    last revision, -2 the previous one). If the negative number is larger
+    than the branch's history, the first revision is returned.
+    examples:
+      revno:1                   -> return the first revision
+      revno:3:/path/to/branch   -> return the 3rd revision of
+                                   the branch '/path/to/branch'
+      revno:-1                  -> The last revision in a branch.
+      -2:http://other/branch    -> The second to last revision in the
+                                   remote branch.
+      -1000000                  -> Most likely the first revision, unless
+                                   your history is very long.
+    """
     prefix = 'revno:'
 
     def _match_on(self, branch, revs):
@@ -293,6 +310,14 @@
 
 
 class RevisionSpec_revid(RevisionSpec):
+    """Selects a revision using the revision id.
+
+    Supply a specific revision id, that can be used to specify any
+    revision id in the ancestry of the branch. 
+    Including merges, and pending merges.
+    examples:
+      revid:aaaa@bbbb-123456789 -> Select revision 'aaa@bbb-123456789'
+    """    
     prefix = 'revid:'
 
     def _match_on(self, branch, revs):
@@ -306,6 +331,14 @@
 
 
 class RevisionSpec_last(RevisionSpec):
+    """Selects the nth revision from the end.
+
+    Supply a positive number to get the nth revision from the end.
+    This is the same as suppling negative numbers to the 'revno:' spec.
+    examples:
+      last:1        -> return the last revision
+      last:3        -> return the revision 2 before the end.
+    """    
 
     prefix = 'last:'
 
@@ -334,6 +367,21 @@
 
 
 class RevisionSpec_before(RevisionSpec):
+    """Selects the parent of the revision specified.
+
+    Supply any revision spec to return the parent of that revision.
+    It is an error to request the parent of the null revision (before:0).
+    This is mostly useful when inspecting revisions that are not in the
+    revision history of a branch.
+
+    examples:
+      before:1913    -> Return the parent of revno 1913 (revno 1912)
+      before:revid:aaaa@bbbb-1234567890  -> return the parent of revision
+                                            aaa@bbb-1234567890
+      bzr diff -r before:revid:aaaa..revid:aaaa
+            -> Find the changes between revision 'aaaa' and its parent.
+               (what changes did 'aaaa' introduce)
+    """
 
     prefix = 'before:'
     
@@ -367,6 +415,7 @@
 
 
 class RevisionSpec_tag(RevisionSpec):
+    """To be implemented."""
     prefix = 'tag:'
 
     def _match_on(self, branch, revs):
@@ -397,6 +446,21 @@
 
 
 class RevisionSpec_date(RevisionSpec):
+    """Selects a revision on the basis of a datestamp.
+
+    Supply a datestamp to select the first revision that matches the date.
+    Date can be 'yesterday', 'today', 'tomorrow' or a YYYY-MM-DD string.
+    Matches the first entry after a given date (either at midnight or
+    at a specified time).
+
+    One way to display all the changes since yesterday would be:
+        bzr log -r date:yesterday..-1
+
+    examples:
+      date:yesterday            -> select the first revision since yesterday
+      date:2006-08-14,17:10:14  -> select the first revision after
+                                   August 14th, 2006 at 5:10pm.
+    """    
     prefix = 'date:'
     _date_re = re.compile(
             r'(?P<date>(?P<year>\d\d\d\d)-(?P<month>\d\d)-(?P<day>\d\d))?'
@@ -405,13 +469,13 @@
         )
 
     def _match_on(self, branch, revs):
-        """
-        Spec for date revisions:
+        """Spec for date revisions:
           date:value
           value can be 'yesterday', 'today', 'tomorrow' or a YYYY-MM-DD string.
           matches the first entry after a given date (either at midnight or
           at a specified time).
 
+          XXX: This doesn't actually work
           So the proper way of saying 'give me all entries for today' is:
               -r date:yesterday..date:today
         """
@@ -467,6 +531,22 @@
 
 
 class RevisionSpec_ancestor(RevisionSpec):
+    """Selects a common ancestor with a second branch.
+
+    Supply the path to a branch to select the common ancestor.
+
+    The common ancestor is the last revision that existed in both
+    branches. Usually this is the branch point, but it could also be
+    a revision that was merged.
+
+    This is frequently used with 'diff' to return all of the changes
+    that your branch introduces, while excluding the changes that you
+    have not merged from the remote branch.
+
+    examples:
+      ancestor:/path/to/branch
+      bzr diff -r ancestor:../../mainline/branch
+    """
     prefix = 'ancestor:'
 
     def _match_on(self, branch, revs):
@@ -493,9 +573,12 @@
 
 
 class RevisionSpec_branch(RevisionSpec):
-    """A branch: revision specifier.
-
-    This takes the path to a branch and returns its tip revision id.
+    """Selects the last revision of a specified branch.
+
+    Supply the path to a branch to select its last revision.
+
+    examples:
+      branch:/path/to/branch
     """
     prefix = 'branch:'
 

=== modified file 'bzrlib/tests/blackbox/test_help.py'
--- bzrlib/tests/blackbox/test_help.py	2006-07-30 07:02:22 +0000
+++ bzrlib/tests/blackbox/test_help.py	2006-09-20 20:59:27 +0000
@@ -32,6 +32,18 @@
                 self.fail("bad output from bzr %s:\n%r" % (cmd, output))
         # see https://launchpad.net/products/bzr/+bug/35940, -h doesn't work
 
+    def test_help_topics(self):
+        """Smoketest for 'bzr help topics'"""
+        out, err = self.run_bzr('help', 'topics')
+        self.assertContainsRe(out, 'basic')
+        self.assertContainsRe(out, 'topics')
+
+    def test_help_revisionspec(self):
+        """Smoke test for 'bzr help revisionspec'"""
+        out, err = self.run_bzr('help', 'revisionspec')
+        self.assertContainsRe(out, 'revno:')
+        self.assertContainsRe(out, 'date:')
+
     def test_help_commands(self):
         dash_help  = self.runbzr('--help commands')[0]
         commands   = self.runbzr('help commands')[0]

=== modified file 'doc/specifying_revisions.txt'
--- doc/specifying_revisions.txt	2006-07-30 06:52:39 +0000
+++ doc/specifying_revisions.txt	2006-09-20 20:59:27 +0000
@@ -5,6 +5,8 @@
 Description of the subarguments for ``-r`` as used by commands such as
 **bzr log** and **bzr merge**.
 
+You can get the online documentation with 'bzr help revisionspec'.
+
 Revision specs and ranges
 =========================
 

=== modified file 'doc/tutorial.txt'
--- doc/tutorial.txt	2006-08-16 13:54:49 +0000
+++ doc/tutorial.txt	2006-09-20 20:59:27 +0000
@@ -107,8 +107,9 @@
 ==============================
 
 Bazaar installs a single new command, **bzr**.  Everything else is a
-subcommand of this.  You can get some help with `bzr help`.  There will be
-more in the future.
+subcommand of this.  You can get some help with `bzr help`. Some arguments 
+are grouped in topics: 'bzr help topics' to see which topics are available.
+There will be more in the future.
 
 One function of a version control system is to keep track of who changed
 what.  In a decentralized system, that requires an identifier for each

